@@ -52,7 +52,6 @@ from sfepy.postprocess.domain_specific import DomainSpecificPlot
 omits = [
     'vibro_acoustic3d_mid.py',
     'linear_elastic_mM.py',
-    'time_poisson_explicit.py',
     'its2D_5.py',
     'linear_elastic_probes.py',
     '__init__.py',
@@ -206,7 +205,7 @@ custom = {
             'rel_scaling' : 0.1,
         },
     },
-    'thermo_elasticity/thermo_elasticity_ess.py' : {
+    'multi_physics/thermo_elasticity_ess.py' : {
         '' : {
             'is_wireframe' : True,
             'only_names' : ['u'],
