@@ -8,9 +8,9 @@ import numpy as nm
 import scipy.sparse as sp
 
 from sfepy.base.base import output, assert_, get_default, iter_dict_of_lists
-from sfepy.base.base import debug, OneTypeList, Container, Struct
+from sfepy.base.base import OneTypeList, Container, Struct
 from sfepy.discrete import Materials, Variables, create_adof_conns
-from sfepy.discrete.fem.extmods.cmesh import create_mesh_graph
+from sfepy.discrete.common.extmods.cmesh import create_mesh_graph
 from sfepy.terms import Terms, Term
 
 def parse_definition(equation_def):
@@ -240,7 +240,8 @@ class Equations(Container):
                           term.region.name, term.arg_str))
 
     def time_update(self, ts, ebcs=None, epbcs=None, lcbcs=None,
-                    functions=None, problem=None, verbose=True):
+                    functions=None, problem=None, active_only=True,
+                    verbose=True):
         """
         Update the equations for current time step.
 
@@ -263,6 +264,10 @@ class Equations(Container):
             The user functions for boundary conditions, materials, etc.
         problem : Problem instance, optional
             The problem that can be passed to user functions as a context.
+        active_only : bool
+            If True, the active DOF connectivities and matrix graph have
+            reduced size and are created with the reduced (active DOFs only)
+            numbering.
         verbose : bool
             If False, reduce verbosity.
 
@@ -276,12 +281,14 @@ class Equations(Container):
         self.variables.time_update(ts, functions, verbose=verbose)
 
         active_bcs = self.variables.equation_mapping(ebcs, epbcs, ts, functions,
-                                                     problem=problem)
-        graph_changed = active_bcs != self.active_bcs
+                                                     problem=problem,
+                                                     active_only=active_only)
+        graph_changed = active_only and (active_bcs != self.active_bcs)
         self.active_bcs = active_bcs
 
         if graph_changed or not self.variables.adof_conns:
-            adcs = create_adof_conns(self.conn_info, self.variables.adi.indx)
+            adcs = create_adof_conns(self.conn_info, self.variables.adi.indx,
+                                     active_only=active_only)
             self.variables.set_adof_conns(adcs)
 
         self.variables.setup_lcbc_operators(lcbcs, ts, functions)
@@ -312,10 +319,11 @@ class Equations(Container):
         self.materials.time_update(ts, self, mode=mode, problem=problem,
                                    verbose=verbose)
 
-    def setup_initial_conditions(self, ics, functions):
+    def setup_initial_conditions(self, ics, functions=None):
         self.variables.setup_initial_conditions(ics, functions)
 
-    def get_graph_conns(self, any_dof_conn=False, rdcs=None, cdcs=None):
+    def get_graph_conns(self, any_dof_conn=False, rdcs=None, cdcs=None,
+                        active_only=True):
         """
         Get DOF connectivities needed for creating tangent matrix graph.
 
@@ -328,6 +336,9 @@ class Equations(Container):
         rdcs, cdcs : arrays, optional
             Additional row and column DOF connectivities, corresponding
             to the variables used in the equations.
+        active_only : bool
+            If True, the active DOF connectivities have reduced size and are
+            created with the reduced (active DOFs only) numbering.
 
         Returns
         -------
@@ -365,29 +376,36 @@ class Equations(Container):
                     or info.is_trace or any_dof_conn):
                 continue
 
-
             rreg_name = info.get_region_name(can_trace=False)
             creg_name = info.get_region_name()
 
-            for rig, cig in info.iter_igs():
-                rname = rvar.get_primary_name()
-                rkey = (rname, rreg_name, dct, rig, False)
-                ckey = (cvar.name, creg_name, dct, cig, info.is_trace)
+            rname = rvar.get_primary_name()
+            rkey = (rname, rreg_name, dct, False)
+            ckey = (cvar.name, creg_name, dct, info.is_trace)
 
-                dc_key = (rkey, ckey)
+            dc_key = (rkey, ckey)
 
-                if not dc_key in shared:
-                    try:
-                        rdcs.append(adcs[rkey])
-                        cdcs.append(adcs[ckey])
-                    except:
-                        debug()
-                    shared.add(dc_key)
+            if not dc_key in shared:
+                rdc = adcs[rkey]
+                cdc = adcs[ckey]
+                if not active_only:
+                    ii = nm.where(rdc < 0)
+                    rdc = rdc.copy()
+                    rdc[ii] = -1 - rdc[ii]
+
+                    ii = nm.where(cdc < 0)
+                    cdc = cdc.copy()
+                    cdc[ii] = -1 - cdc[ii]
+
+                rdcs.append(rdc)
+                cdcs.append(cdc)
+
+                shared.add(dc_key)
 
         return rdcs, cdcs
 
     def create_matrix_graph(self, any_dof_conn=False, rdcs=None, cdcs=None,
-                            shape=None, verbose=True):
+                            shape=None, active_only=True, verbose=True):
         """
         Create tangent matrix graph, i.e. preallocate and initialize the
         sparse storage needed for the tangent matrix. Order of DOF
@@ -406,6 +424,9 @@ class Equations(Container):
             The required shape, if it is different from the shape
             determined by the equations variables. This may be needed if
             additional row and column DOF connectivities are passed in.
+        active_only : bool
+            If True, the matrix graph has reduced size and is created with the
+            reduced (active DOFs only) numbering.
         verbose : bool
             If False, reduce verbosity.
 
@@ -427,7 +448,8 @@ class Equations(Container):
             return None
 
         rdcs, cdcs = self.get_graph_conns(any_dof_conn=any_dof_conn,
-                                          rdcs=rdcs, cdcs=cdcs)
+                                          rdcs=rdcs, cdcs=cdcs,
+                                          active_only=active_only)
 
         if not len(rdcs):
             output('no matrix (empty dof connectivities)!')
@@ -783,10 +805,10 @@ class Equation(Struct):
         """
         Parameters
         ----------
-        mode : one of 'eval', 'el_avg', 'qp', 'weak'
+        mode : one of 'eval', 'el_eval', 'el_avg', 'qp', 'weak'
             The evaluation mode.
         """
-        if mode == 'eval':
+        if mode in ('eval', 'el_eval', 'el_avg', 'qp'):
             val = 0.0
             for term in self.terms:
                 aux, status = term.evaluate(mode=mode,
@@ -797,21 +819,6 @@ class Equation(Struct):
 
             out = val
 
-        elif mode in ('el_avg', 'el', 'qp'):
-
-            vals = []
-            for term in self.terms:
-                val, iels, status = term.evaluate(mode=mode,
-                                                  term_mode=term_mode,
-                                                  standalone=False,
-                                                  ret_status=True)
-                vals.append(val)
-
-            if len(vals) == 1:
-                vals = vals[0]
-
-            out = vals
-
         elif mode == 'weak':
 
             if dw_mode == 'vector':
