@@ -195,6 +195,18 @@ def get_opacities(opacity):
 
     return opacities
 
+def set_colormap(source, colormap_name):
+    """
+    Set the given colormap to all look-up tables depending on the given source.
+    """
+    if hasattr(source, 'scalar_lut_manager'):
+        source.scalar_lut_manager.lut_mode = colormap_name
+        source.vector_lut_manager.lut_mode = colormap_name
+
+    elif hasattr(source, 'children'):
+        for child in source.children:
+            set_colormap(child, colormap_name)
+
 class Viewer(Struct):
     """
     Class to automate visualization of various data using Mayavi.
@@ -664,6 +676,8 @@ class Viewer(Struct):
                 surf.actor.property.representation = 'wireframe'
                 surf.actor.mapper.scalar_visibility = False
 
+        set_colormap(source, self.colormap)
+
     def render_scene(self, scene, options):
         """
         Render the scene, preferably after it has been activated.
@@ -673,6 +687,7 @@ class Viewer(Struct):
         self.build_mlab_pipeline(**options)
         self.source.update() # Force source update to see e.g. streamlines.
         scene.scene.reset_zoom()
+        scene.scene.camera.parallel_projection = options['parallel_projection']
 
         view = options['view']
         if view is None:
@@ -718,8 +733,9 @@ class Viewer(Struct):
         pass
 
     def call_mlab(self, scene=None, show=True, is_3d=False,
-                  view=None, roll=None,
+                  view=None, roll=None, parallel_projection=False,
                   fgcolor=(0.0, 0.0, 0.0), bgcolor=(1.0, 1.0, 1.0),
+                  colormap='blue-red',
                   layout='rowcol', scalar_mode='iso_surface',
                   vector_mode='arrows_norm', rel_scaling=None, clamping=False,
                   ranges=None, is_scalar_bar=False, is_wireframe=False,
@@ -745,12 +761,16 @@ class Viewer(Struct):
             `mlab.view()`.
         roll : float
             Roll angle tuple as in mlab.roll().
+        parallel_projection: bool
+            If True, use parallel projection.
         fgcolor : tuple of floats (R, G, B)
             The foreground color, that is the color of all text
             annotation labels (axes, orientation axes, scalar bar
             labels).
         bgcolor : tuple of floats (R, G, B)
             The background color.
+        colormap : str
+            The colormap name.
         layout : str
             Grid layout for placing the datasets. Possible values are:
             'row', 'col', 'rowcol', 'colrow'.
@@ -806,6 +826,7 @@ class Viewer(Struct):
         """
         self.fgcolor = fgcolor
         self.bgcolor = bgcolor
+        self.colormap = colormap
 
         if filter_names is None:
             filter_names = ['node_groups', 'mat_id']
@@ -891,8 +912,8 @@ class Viewer(Struct):
         self.file_source = create_file_source(self.filename, watch=self.watch,
                                               offscreen=self.offscreen)
         steps, times = self.file_source.get_ts_info()
-        has_several_times = len(times) > 0
-        has_several_steps = has_several_times or (len(steps) > 0)
+        has_several_times = len(times) > 1
+        has_several_steps = has_several_times or (len(steps) > 1)
 
         if gui is not None:
             gui.has_several_steps = has_several_steps
